@@ -45,9 +45,8 @@
 
 #include "urldata.h"
 #include "cfilters.h"
-#include "sendf.h"
+#include "curl_trc.h"
 #include "hostip.h"
-#include "hash.h"
 #include "url.h"
 #include "curlx/inet_pton.h"
 #include "connect.h"
